@@ -19,6 +19,6 @@
 from __future__ import (absolute_import, division, print_function)
 __metaclass__ = type
 
-__version__ = '2.7.1'
+__version__ = '2.8.12'
 __author__ = 'Ansible, Inc.'
-__codename__ = 'In the Light'
+__codename__ = 'How Many More Times'
