@@ -10,8 +10,9 @@ def main():
         'test/sanity/code-smell/%s' % os.path.basename(__file__),
         'lib/ansible/module_utils/six/__init__.py',
         'lib/ansible/module_utils/urls.py',
-        'test/units/module_utils/urls/test_open_url.py',
+        'test/units/module_utils/urls/test_Request.py',
         'test/units/module_utils/urls/test_fetch_url.py',
+        'test/utils/shippable/check_matrix.py',
     ])
 
     for path in sys.argv[1:] or sys.stdin.read().splitlines():
