@@ -6,10 +6,9 @@ import sys
 
 
 def main():
-    allowed = set([
+    standard_shebangs = set([
         b'#!/bin/bash -eu',
         b'#!/bin/bash -eux',
-        b'#!/bin/bash',
         b'#!/bin/sh',
         b'#!/usr/bin/env bash',
         b'#!/usr/bin/env fish',
@@ -18,6 +17,12 @@ def main():
         b'#!/usr/bin/make -f',
     ])
 
+    integration_shebangs = set([
+        b'#!/bin/sh',
+        b'#!/usr/bin/env bash',
+        b'#!/usr/bin/env python',
+    ])
+
     module_shebangs = {
         '': b'#!/usr/bin/python',
         '.py': b'#!/usr/bin/python',
@@ -25,10 +30,28 @@ def main():
     }
 
     skip = set([
+        'test/integration/targets/collections/collection_root_user/ansible_collections/testns/testcoll/plugins/modules/win_csbasic_only.ps1',
+        'test/integration/targets/collections/collection_root_user/ansible_collections/testns/testcoll/plugins/modules/win_selfcontained.ps1',
+        'test/integration/targets/collections/collection_root_user/ansible_collections/testns/testcoll/plugins/modules/win_uses_coll_csmu.ps1',
+        'test/integration/targets/collections/collection_root_user/ansible_collections/testns/testcoll/plugins/modules/win_uses_coll_psmu.ps1',
         'test/integration/targets/win_module_utils/library/legacy_only_new_way_win_line_ending.ps1',
         'test/integration/targets/win_module_utils/library/legacy_only_old_way_win_line_ending.ps1',
+        'test/utils/shippable/timing.py',
+        'test/integration/targets/old_style_modules_posix/library/helloworld.sh',
+        'test/integration/targets/gathering_facts/library/bogus_facts',
+        # Python 3-only.  Only run by release engineers
+        'hacking/release-announcement.py',
     ])
 
+    # see https://unicode.org/faq/utf_bom.html#bom1
+    byte_order_marks = (
+        (b'\x00\x00\xFE\xFF', 'UTF-32 (BE)'),
+        (b'\xFF\xFE\x00\x00', 'UTF-32 (LE)'),
+        (b'\xFE\xFF', 'UTF-16 (BE)'),
+        (b'\xFF\xFE', 'UTF-16 (LE)'),
+        (b'\xEF\xBB\xBF', 'UTF-8'),
+    )
+
     for path in sys.argv[1:] or sys.stdin.read().splitlines():
         if path in skip:
             continue
@@ -42,9 +65,15 @@ def main():
                 if executable:
                     print('%s:%d:%d: file without shebang should not be executable' % (path, 0, 0))
 
+                for mark, name in byte_order_marks:
+                    if shebang.startswith(mark):
+                        print('%s:%d:%d: file starts with a %s byte order mark' % (path, 0, 0, name))
+                        break
+
                 continue
 
             is_module = False
+            is_integration = False
 
             if path.startswith('lib/ansible/modules/'):
                 is_module = True
@@ -57,6 +86,8 @@ def main():
 
                 continue
             elif path.startswith('test/integration/targets/'):
+                is_integration = True
+
                 dirname = os.path.dirname(path)
 
                 if dirname.endswith('/library') or dirname in (
@@ -82,6 +113,11 @@ def main():
                 else:
                     print('%s:%d:%d: expected module extension %s but found: %s' % (path, 0, 0, expected_ext, ext))
             else:
+                if is_integration:
+                    allowed = integration_shebangs
+                else:
+                    allowed = standard_shebangs
+
                 if shebang not in allowed:
                     print('%s:%d:%d: unexpected non-module shebang: %s' % (path, 1, 1, shebang))
 
